/* hashinterface.h
 *
 * hashkill - a hash cracking tool
 * Copyright (C) 2010 Milen Rangelov <gat3way@gat3way.eu>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#ifndef HASHINTERFACE_H
#define HASHINTERFACE_H


/* Do not change those unless you know what you are doing */
#define HASHKILL_VERSION "0.2.3"			// hashkill version
#define HASHKILL_MAXTHREADS (unsigned int)64		// maximum threads
#define HASHKILL_MAXQUEUESIZE 1024 			// maximum queue size
#define HASHFILE_MAX_LINE_LENGTH 128			// maximum line length
#define HASHFILE_MAX_PLAIN_LENGTH 16
#define VECTORSIZE 128

#define THREAD_LENPROVIDED 1000
#define THREAD_SALTPROVIDED 20000

/* thread queues */
typedef struct hash_lifo_node_s
{
    char *plaintext[VECTORSIZE];
    int lens[VECTORSIZE];
} hash_lifo[HASHKILL_MAXQUEUESIZE];

hash_lifo hash_queue[HASHKILL_MAXTHREADS];


/* hashes linked lists */
struct hash_list_s
{
    char *username;
    char *hash;
    char *salt;
    char *salt2;
    struct hash_list_s *prev;
    struct hash_list_s *next;
} *hash_list, *cracked_list;


/* Attack method enumeration */
typedef enum attack_method_e
{
    attack_method_dictionary,
    attack_method_simple_bruteforce,
    attack_method_hybrid,
    attack_method_markov
} attack_method_t;


attack_method_t attack_method;
int hash_ret_len;

/* Global variables needed by modules */
char hashlist_file[255];
char hash_dictionary_file[255];
int bruteforce_start;
int bruteforce_end;
char bruteforce_charset[255];
char bruteforce_prefix[64];
char bruteforce_suffix[64];
int hashes_count;
int session_restore_flag;
char markov_statfile[255];
int markov_max_len;
int markov_threshold;
int markov0[88];
int markov1[88][88];
char *markov_charset;
int hash_is_raw;
int vectorsize;


/* List manipulation routines */
hash_stat add_hash_list(char *username, char *hash, char *salt, char *salt2);
hash_stat add_cracked_list(char *username, char *hash, char *salt, char *salt2);
hash_stat del_hash_list(struct hash_list_s *node);
hash_stat del_cracked_list(struct hash_list_s *node);


/* Get information about linked lists */
int get_cracked_num(void);
void print_hash_list(void);
int get_hashes_num(void);
void print_cracked_list(void);
void print_cracked_list_to_file(char *filename);


/* Proto callback functions that are registered to the plugin library */
void hash_proto_add_username(const char *username);
void hash_proto_add_hash(const char *hash, int len);
void hash_proto_add_salt(const char *salt);
void hash_proto_add_salt2(const char *salt2);
void hash_proto_md5(char *plaintext[VECTORSIZE], char *hashmd5[VECTORSIZE], int len, int threadid);
void hash_proto_md5_unicode(char *plaintext[VECTORSIZE], char *hashmd5[VECTORSIZE], int len[VECTORSIZE]);
void hash_proto_md5_unicode_slow(char *plaintext[VECTORSIZE], char *hashmd5[VECTORSIZE], int len[VECTORSIZE]);
void hash_proto_md5_slow(char *plaintext[VECTORSIZE], char *hashmd5[VECTORSIZE], int len, int threadid);
void hash_proto_md4(char *plaintext[VECTORSIZE], char *hashmd5[VECTORSIZE], int len[VECTORSIZE]);
void hash_proto_ripemd160(const char *plaintext[VECTORSIZE], char *hashmd5[VECTORSIZE], int len[VECTORSIZE]);
void hash_proto_md5_hex(const char *hash[VECTORSIZE],  char *hashhex[VECTORSIZE]);
void hash_proto_sha1(char *plaintext[VECTORSIZE], char *hashmd5[VECTORSIZE], int len, int threadid);
void hash_proto_sha1_unicode(char *plaintext[VECTORSIZE], char *hashmd5[VECTORSIZE], int len[VECTORSIZE]);
void hash_proto_sha1_slow(char *plaintext[VECTORSIZE], char *hashmd5[VECTORSIZE], int len[VECTORSIZE]);
void hash_proto_sha1_hex(const char *hash[VECTORSIZE], char *hashhex[VECTORSIZE]);
void hash_proto_sha256_unicode(const char *plaintext[VECTORSIZE], char *hashmd5[VECTORSIZE], int len[VECTORSIZE]);
void hash_proto_sha256_hex(const char *hash[VECTORSIZE], char *hashhex[VECTORSIZE]);
void hash_proto_sha512_unicode(const char *plaintext[VECTORSIZE], char *hashmd5[VECTORSIZE], int len[VECTORSIZE]);
void hash_proto_sha512_hex(const char *hash[VECTORSIZE], char *hashhex[VECTORSIZE]);
void hash_proto_fcrypt(const char *password[VECTORSIZE], const char *salt, char *ret[VECTORSIZE]);
void hash_proto_new_biomem(FILE *file);
void hash_proto_PEM_readfile(const char *passphrase, int *RSAret);
void hash_proto_pbkdf2(const char *pass, unsigned char *salt, int saltlen, int iter, int keylen, unsigned char *out);
void hash_proto_hmac_sha1_file(void *key, int keylen, char *filename, long offset, long size, unsigned char *output, int outputlen);
void hash_proto_pbkdf512(const char *pass, unsigned char *salt, int saltlen, int iter, int keylen, unsigned char *out);
void hash_proto_aes_encrypt(const unsigned char *key, int keysize, const unsigned char *in, int len, unsigned char *vec, unsigned char *out, int mode);
void hash_proto_aes_decrypt(const unsigned char *key, int keysize, const unsigned char *in, int len, unsigned char *vec, unsigned char *out, int mode);
void hash_proto_des_ecb_encrypt(const unsigned char *key, int keysize, const unsigned char *in[VECTORSIZE], int len, unsigned char *out[VECTORSIZE], int mode);
void hash_proto_des_ecb_decrypt(const unsigned char *key, int keysize, const unsigned char *in, int len, unsigned char *out, int mode);
void hash_proto_des_cbc_encrypt(const unsigned char *key[VECTORSIZE], int keysize, const unsigned char *in[VECTORSIZE], int len[VECTORSIZE], unsigned char *out[VECTORSIZE], unsigned char *iv[VECTORSIZE], int mode);
void hash_proto_lm(const unsigned char *in[VECTORSIZE], unsigned char *out[VECTORSIZE]);


/* cleanup routines */
void cleanup_lists(void);

/* Markov attack routines */
void markov_attack_init(void);
void markov_print_statfiles(void);
hash_stat markov_load_statfile(char *statname);

#endif
