/* lm.c
 *
 * hashkill - a hash cracking tool
 * Copyright (C) 2010 Milen Rangelov <gat3way@gat3way.eu>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <alloca.h>
#include "plugin.h"
#include "err.h"
#include "hashinterface.h"


int vectorsize;

char * hash_plugin_summary(void)
{
    return("lm \t\tLM plugin");
}


char * hash_plugin_detailed(void)
{
    return("lm - LM plugin\n"
	    "------------------------\n"
	    "Use this module to crack simple LM hashes\n"
	    "Input should be in form: \'user:hash\'\n"
	    "Known software that uses this password hashing method:\n"
	    "Older Microsoft Windows versions\n"
	    "\nAuthor: Milen Rangelov <gat3way@gat3way.eu>\n");
}


hash_stat hash_plugin_parse_hash(char *hashline, char *filename)
{
    char username[HASHFILE_MAX_LINE_LENGTH];
    char *hash = alloca(255);
    char *hash2 = alloca(33);

    char line[HASHFILE_MAX_LINE_LENGTH];
    char *temp_str;
    
    if (!hashline) return hash_err;
    
    if (strlen(hashline)<2) return hash_err;
    
    snprintf(line, HASHFILE_MAX_LINE_LENGTH-1, "%s", hashline);
    strcpy(username, strtok(line, ":"));
    temp_str=strtok(NULL,":");
    if (temp_str) 
    {
	strcpy(hash2, temp_str);
    }

    if (strlen(hash2)!=32)
    {
	return hash_err;
    }

    hash = strlow(hash2);
    
    
    (void)hash_add_username(username);

    hex2str(line, hash, 32);
    (void)hash_add_hash(line,16);

    (void)hash_add_salt("");
    (void)hash_add_salt2("");

    return hash_ok;
}


hash_stat hash_plugin_check_hash(const char *hash, const char *password[VECTORSIZE], const char *salt,  char *salt2[VECTORSIZE], const char *username, int *num, int threadid)
{
    unsigned char *keyl[VECTORSIZE];
    char *newp[VECTORSIZE];
    int i,j;
    
    for (j=0;j<vectorsize;j++)
    {
	newp[j] = strupr((char *)password[j]);
	keyl[j] = alloca(8);

	keyl[j][0] = newp[j][0]>>1;
	keyl[j][1] = ((newp[j][0]&0x01)<<6) | (newp[j][1]>>2);
	keyl[j][2] = ((newp[j][1]&0x03)<<5) | (newp[j][2]>>3);
	keyl[j][3] = ((newp[j][2]&0x07)<<4) | (newp[j][3]>>4);
	keyl[j][4] = ((newp[j][3]&0x0F)<<3) | (newp[j][4]>>5);
        keyl[j][5] = ((newp[j][4]&0x1F)<<2) | (newp[j][5]>>6);
	keyl[j][6] = ((newp[j][5]&0x3F)<<1) | (newp[j][6]>>7);
	keyl[j][7] = newp[j][6]&0x7F;

	keyl[j][8] = newp[j][7]>>1;
	keyl[j][9] = ((newp[j][7]&0x01)<<6) | (newp[j][8]>>2);
	keyl[j][10] = ((newp[j][8]&0x03)<<5) | (newp[j][9]>>3);
	keyl[j][11] = ((newp[j][9]&0x07)<<4) | (newp[j][10]>>4);
	keyl[j][12] = ((newp[j][10]&0x0F)<<3) | (newp[j][11]>>5);
	keyl[j][13] = ((newp[j][11]&0x1F)<<2) | (newp[j][12]>>6);
	keyl[j][14] = ((newp[j][12]&0x3F)<<1) | (newp[j][13]>>7);
	keyl[j][15] = newp[j][13]&0x7F;
	for (i=0;i<16;i++) 
	{
    	    keyl[j][i] = (keyl[j][i]<<1);
	}

    }
    hash_lm((const unsigned char **)keyl, (unsigned char **)salt2);

    return hash_ok;
}


int hash_plugin_hash_length(void)
{
    return 0;
}

int hash_plugin_is_raw(void)
{
    return 1;
}

int hash_plugin_is_special(void)
{
    return 0;
}

void get_vector_size(int size)
{
    vectorsize = size;
}
