/* rsa-privkey.c
 *
 * hashkill - a hash cracking tool
 * Copyright (C) 2010 Milen Rangelov <gat3way@gat3way.eu>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <alloca.h>
#include <fcntl.h>
#include <sys/types.h>

#include "zlib.h"

#include "plugin.h"
#include "err.h"
#include "hashinterface.h"

#define CHECK_BIT(var,pos) ((var) & (1<<(pos)))

int vectorsize;


char myfilename[255];
FILE *myfile;
unsigned int g_CrcTable[256];


unsigned char winzip_salt[16];
int has_strong_encryption = 0;
int has_winzip_encryption = 0;
int has_ext_flag = 0;
int winzip_key_size = 0;
int winzip_salt_size;
unsigned char winzip_check[2];
unsigned char zip_normbuf[12];
unsigned char zip_crc32[4];
unsigned char zip_tim[2];
long fileoffset;
int filenamelen;
int comprsize, ucomprsize;

#define kCrcPoly 0xEDB88320
#define CRC_UPDATE_BYTE(crc, b) (g_CrcTable[((crc) ^ (b)) & 0xFF] ^ ((crc) >> 8))
static void  CrcGenerateTable(void)
{
  unsigned int i;
  for (i = 0; i < 256; i++)
  {
    unsigned int r = i;
    int j;
    for (j = 0; j < 8; j++)
      r = (r >> 1) ^ (kCrcPoly & ~((r & 1) - 1));
    g_CrcTable[i] = r;
  }
}



char * hash_plugin_summary(void)
{
    return("zip \t\tZIP passwords plugin");
}


char * hash_plugin_detailed(void)
{
    return("zip - A ZIP passwords plugin\n"
	    "------------------------------------------------\n"
	    "Use this module to crack ZIP archives passwords\n"
	    "Input should be a passworded ZIP file specified with -f\n"
	    "Supports the old encryption method as well as AES encryption (WinZIP)\n"
	    "Known software that uses this password hashing method:\n"
	    "WinZIP, p7zip, etc\n"
	    "\nAuthor: Milen Rangelov <gat3way@gat3way.eu>\n");
}


hash_stat hash_plugin_parse_hash(char *hashline, char *filename)
{
    int fd;
    char buf[4096];
    unsigned int u321;
    unsigned short u161, genpurpose, extrafieldlen;
    
    fileoffset = 0;
    CrcGenerateTable();
    strcpy(myfilename, filename);

    fd = open(filename, O_RDONLY);
    if (fd<1)
    {
	elog("Cannot open file %s\n", filename);
	return hash_err;
    }
    read(fd, &u321, 4);
    fileoffset+=4;
    if (u321 != 0x04034b50)
    {
	elog("Not a ZIP file: %s!\n", filename);
	return hash_err;
    }
    /* version needed to extract */
    read(fd, &u161, 2);
    fileoffset+=2;
    /* general purpose bit flag */
    read(fd, &genpurpose, 2);
    fileoffset+=2;
    /* compression method, last mod file time, last mod file date */
    read(fd, &u161, 2);
    fileoffset+=2;
    if (u161 == 99) 
    {
	has_winzip_encryption = 1;
    }
    read(fd, &zip_tim, 2);
    read(fd, &u161, 2);
    fileoffset+=4;

    /* crc32 */
    read(fd, zip_crc32, 4);
    fileoffset+=4;

    /* compressed size */
    read(fd, &comprsize, 4);
    fileoffset+=4;

    /* uncompressed size */
    read(fd, &ucomprsize, 4);
    fileoffset+=4;

    /* file name length */
    read(fd, &filenamelen, 2);
    fileoffset+=2;

    /* extra field length */
    read(fd, &extrafieldlen, 2);
    fileoffset+=2;

    read(fd, buf, filenamelen);
    fileoffset+=filenamelen;


    /* extra field should be taken care if winzip encryption is used */
    read(fd, buf, extrafieldlen);
    fileoffset+=extrafieldlen;



    if (has_winzip_encryption == 1) 
    {
	switch (buf[8])
	{
	    case 1: winzip_key_size = 128;winzip_salt_size = 8;break;
	    case 2: winzip_key_size = 192;winzip_salt_size = 12;break;
	    case 3: winzip_key_size = 256;winzip_salt_size = 16;break;
	    default: elog("Unknown WinZIP AES encryption key length?!?\n%s","");return hash_err;
	}
    }
    /* check if bit 0 in genpurpose are set => we've got strong encryption */
    if (CHECK_BIT(genpurpose, 0) == 0)
    {
	elog("Zip file: %s is not encrypted\n", filename);
	return hash_err;
    }


    if (CHECK_BIT(genpurpose, 3) == 0)
    {
	has_ext_flag = 1;
    }

    
    /* Parse the encryption header - the winzip way */
    if (has_winzip_encryption)
    {
	hlog("File %s: WinZIP AES encryption used, key size: %dbits\n", filename, winzip_key_size);
	read(fd, winzip_salt, winzip_salt_size);
	read(fd, winzip_check, 2);
    }
    else
    {
	hlog("File %s: Older (<=2.0) encryption used\n", filename);
	read(fd, (char *)&zip_normbuf, 12);
	fileoffset+=12;

    }
    

    close(fd);
    
    (void)hash_add_username(filename);
    (void)hash_add_hash("ZIP file        ",0);
    (void)hash_add_salt("123");
    (void)hash_add_salt2("                              ");

    return hash_ok;
}




hash_stat hash_plugin_check_hash(const char *hash, const char *password[VECTORSIZE], const char *salt,  char * salt2[VECTORSIZE], const char *username, int *num, int threadid)
{
    unsigned char key[68];
    unsigned char check[2];
    unsigned long key0,key1,key2;
    unsigned char norm_zip_local[12];
    int fd;
    int ret, bsize, rsize, usize;
    unsigned char in[1024*16+100];
    unsigned char out[1024*16*10+100];
    int localcrc32;
    unsigned char authcode[10];
    unsigned char authresult[10];

    
    if (has_winzip_encryption==1)
    {
	hash_pbkdf2(password[0], winzip_salt, winzip_salt_size, 1000, 2*(winzip_key_size/8)+2, key);
	check[0] = key[2*(winzip_key_size/8)];
	check[1] = key[2*(winzip_key_size/8)+1];
	
	/* As mentioned in WinZIP documentation, this gives out 1/65535 error probability. Calculate auth codes */
        if (memcmp(winzip_check, check, 2)==0) 
	{
	    fd = open(myfilename, O_RDONLY);
	    lseek(fd, fileoffset + comprsize - 10, SEEK_SET);
	    read(fd, authcode, 10);
	    hash_hmac_sha1_file((unsigned char *)&key[winzip_key_size/8], winzip_key_size/8, myfilename, fileoffset+winzip_salt_size+2, comprsize-12-winzip_salt_size, (unsigned char *)&authresult, 10);
/*	    printf("authresult = ");
	    for (ret=0;ret<10;ret++) printf("%02x ",authresult[ret]);
	    printf("\nauthcode = ");
	    for (ret=0;ret<10;ret++) printf("%02x ",authcode[ret]);
	    printf("\n");*/
	    if (memcmp((char *)&authresult[4], (char *)&authcode[4], 6)==0)
	    {
		*num=0;
		memcpy(salt2[0],"ZIP file        \0\0",17);
		return hash_ok;
	    }
	    else
	    {
		salt2[0][0]=password[0][0];
		return hash_err;
	    }
	}
	else 
	{
	    salt2[0][0]=password[0][0];
	    return hash_err;
	}
    }
    else 
    {
	key0 = 305419896L;
	key1 = 591751049L;
	key2 = 878082192L;
	int i;
	//unsigned long crc_32( int func, const char *buf, size_t len, unsigned long *crcmerk )
	for (i=0; i<strlen(password[0]); i++)
	{
	    key0=CRC_UPDATE_BYTE(key0, (char)*(password[0]+i));
	    key1 += key0 & 0xff;
	    key1 = key1 * 134775813L + 1;
	    key2 = CRC_UPDATE_BYTE(key2,(char)(key1>>24));
	}
        unsigned char temp1;
        unsigned char c;
        unsigned  long temp;
	memcpy((char *)&norm_zip_local, (char *)&zip_normbuf, 12);
	for (i=0;i<12;i++)
	{

	    temp = (key2) | 2;
	    temp1 = (((temp * (temp ^1)) >> 8));
	    c = norm_zip_local[i] ^ temp1;
	    
	    key0 = CRC_UPDATE_BYTE(key0,c);
	    key1 += key0 & 0xff;
	    key1 = key1 * 134775813L + 1;
	    key2 = CRC_UPDATE_BYTE(key2,(char)(key1 >> 24));

	    norm_zip_local[i] = c;
	}
	
//	if (strlen(password)>0)hlog("zcrc3=%02x ztim=%02x nzl=%02x pwd=%s\n",zip_crc32[3], zip_tim[1],  norm_zip_local[11], password);
	if ( ((has_ext_flag==0) && (zip_tim[1] == norm_zip_local[11]))
	    || ((has_ext_flag==1) && (zip_crc32[3] == norm_zip_local[11])))
	{

	    fd = open(myfilename,O_RDONLY);
	    lseek(fd, fileoffset, SEEK_SET);
	    z_stream strm;
	    strm.zalloc = Z_NULL;
    	    strm.zfree = Z_NULL;
    	    strm.opaque = Z_NULL;
            strm.avail_in = 1024*16;
            strm.avail_out = 1024*16*10;

            strm.next_in = in;
            strm.next_out = out;

            ret = inflateInit2(&strm,-15);
            if (ret != Z_OK) elog("inflateinit ERROR!\n%s","");
	    localcrc32 = crc32(0, Z_NULL,0);
            rsize = 0;usize = 0;
            while (rsize < (comprsize-12))
            {
        	if ((comprsize-rsize)>1024*16) ret = 1024*16;
		else ret = comprsize-rsize;
        	bsize = read(fd, in,  ret);

        	for (ret = 0; ret < bsize;ret++)
        	{
        	    temp = (key2) | 2;
		    temp1 = (((temp * (temp ^1)) >> 8));
		    c = in[ret] ^ temp1;
		    key0=CRC_UPDATE_BYTE(key0, c);
		    key1 += key0 & 0xff;
		    key1 = key1 * 134775813L + 1;
		    key2 = CRC_UPDATE_BYTE(key2,(char)(key1>>24));
		    in[ret] = c;
        	}
        	strm.next_in = in;
        	strm.avail_out = bsize*10;
        	strm.avail_in = bsize;
        	strm.next_out = out;
        	usize = strm.total_in;
        	ret = inflate(&strm, Z_SYNC_FLUSH);
        	lseek(fd, fileoffset + strm.total_in,SEEK_SET);
		rsize += (strm.total_in - usize);
        	if (ret == Z_DATA_ERROR) 
        	{
        	    close(fd);
        	    inflateEnd(&strm);
        	    return hash_err;
    		}
                if (ret == Z_NEED_DICT) 
	        {

        	    close(fd);
        	    inflateEnd(&strm);
        	    return hash_err;
    		}
        	if (ret == Z_STREAM_ERROR) 
        	{
        	    close(fd);
        	    inflateEnd(&strm);
        	    return hash_err;
    		}

        	if  ((ret == Z_MEM_ERROR))
		{
		    close(fd);
		    inflateEnd(&strm);
        	    return hash_err;
    		}
    		if (ret == Z_STREAM_END) rsize = comprsize+1;
	    }
	    if (has_ext_flag == 0)
	    {
		if ((zip_tim[1] == norm_zip_local[11]) && (ucomprsize==strm.total_out))
		{
		    memcpy(salt2[0],"ZIP file        \0\0\0\0\0\0\0\0\0",20);
		    *num=0;
		    inflateEnd(&strm);
		    close(fd);
		    return hash_ok;
    		}
    	    }
    	    else if ((norm_zip_local[11] == zip_crc32[3]) && (ucomprsize==strm.total_out))
	    {
		    memcpy(salt2[0],"ZIP file        \0\0\0\0\0\0\0\0\0",20);
		    *num=0;
		    inflateEnd(&strm);
		    close(fd);
		    return hash_ok;
    	    }

	    else
	    {
		salt2[0][0]=password[0][0];
		inflateEnd(&strm);
		close(fd);
		return hash_err;
	    }
    
	close(fd);
	}
    
    }
    return hash_err;
}


int hash_plugin_hash_length(void)
{
    return 16;
}

int hash_plugin_is_raw(void)
{
    return 1;
}

int hash_plugin_is_special(void)
{
    return 1;
}

void get_vector_size(int size)
{
   vectorsize = size;
}
