/* sessions.c
 *
 * hashkill - a hash cracking tool
 * Copyright (C) 2010 Milen Rangelov <gat3way@gat3way.eu>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#define _GNU_SOURCE

#include <stdio.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdlib.h>
#include <stdint.h>
#include <dirent.h>
#include <pwd.h>
#include <openssl/evp.h>
#include <openssl/bio.h>
#include <openssl/buffer.h>
#include "err.h"
#include "threads.h"
#include "hashinterface.h"
#include "plugins.h"
#include "loadfiles.h"

/* Global variables */
static char *session_argv[30];		// should be a copy of main() argv


/* Function prototypes */
hash_stat session_init_file(FILE **sessionfile);
void session_close_file(FILE *sessionfile);
void session_unlink_file();
hash_stat session_write_hashlist(FILE *sessionfile);
hash_stat session_write_crackedlist(FILE *sessionfile);
hash_stat session_write_dictionary_parm(long filepos, char *dictfile, FILE *sessionfile);
hash_stat session_write_bruteforce_parm(int start, int end, char *prefix, char *suffix, char *charset, int curlen, char *curstr, uint64_t progress, FILE *sessionfile);
hash_stat session_write_hybrid_parm(char *dictfile, int start, int end, char *charset, long current_pos, char *current_str, FILE *sessionfile);
void session_put_commandline(char *argv[]);
hash_stat session_write_parameters(char *plugin, attack_method_t attacktype, uint64_t progress, FILE *sessionfile);
hash_stat print_sessions_summary(void);
hash_stat print_session_detailed(char *sessionname);
hash_stat session_restore(char *sessionname);

/* OpenSSL's base64 routines wrapped - cause I'm too lazy to write my own*/
char *base64(const unsigned char *input, int length); 
char *unbase64(unsigned char *input, int length);




char *base64(const unsigned char *input, int length)
{
  BIO *bmem, *b64;
  BUF_MEM *bptr;

  b64 = BIO_new(BIO_f_base64());
  BIO_set_flags(b64, BIO_FLAGS_BASE64_NO_NL);
  bmem = BIO_new(BIO_s_mem());
  b64 = BIO_push(b64, bmem);
  BIO_write(b64, input, length);
  BIO_flush(b64);
  BIO_get_mem_ptr(b64, &bptr);

  char *buff = (char *)malloc(bptr->length+10);
  memcpy(buff, bptr->data, bptr->length);
  buff[bptr->length] = 0;

  BIO_free_all(b64);

  return buff;
}


char *unbase64(unsigned char *input, int length)
{
  BIO *b64, *bmem;

  char *buffer = (char *)malloc(length+10);
  memset(buffer, 0, length);

  b64 = BIO_new(BIO_f_base64());
  BIO_set_flags(b64, BIO_FLAGS_BASE64_NO_NL);
  bmem = BIO_new_mem_buf(input, length);
  bmem = BIO_push(b64, bmem);

  BIO_read(bmem, buffer, length);

  BIO_free_all(bmem);

  return buffer;
}



/* init session file */
hash_stat session_init_file(FILE **sessionfile)
{

/*
    uid_t uid;
    struct passwd *pwd;
    char fname[255];
    int fd;
    struct stat filest;

    uid = getuid();
    pwd = getpwuid(uid);
    if (!pwd)
    {
	elog("Cannot get username for uid %d (%s)\n", (int)uid, strerror(errno));
	return hash_err;
    }
    snprintf(fname, 255, "/usr/share/hashkill/sessions/%s-%d.session", pwd->pw_name, getpid());
    fd = open(fname, O_WRONLY | O_NOFOLLOW | O_CREAT | O_TRUNC, 0640);
    *sessionfile = fdopen(fd, "w");
    if (fstat(fd, &filest) != 0) 
    {
	printf("\n");
	elog("Cannot stat session file :%s - probably a symlink attack!!!\n", fname);
	return hash_err;
    }
    if (filest.st_nlink > 1) 
    {
	printf("\n");
	elog("Hardlink attack attempted! File: %s Owner UID: %d! Please remove the hardlink manually!\n", fname, filest.st_uid);
	fclose(*sessionfile);
	return hash_err;
    }
    if (!(*sessionfile))
    {
	printf("\n");
	elog("Cannot open session file %s for writing (%s)!\n", fname, strerror(errno));
	return hash_err;
    }
    endpwent();
*/

    char fname[255];
    int fd;
    struct stat filest;

    snprintf(fname, 255, "/usr/share/hashkill/sessions/session.tmp");
    fd = open(fname, O_WRONLY | O_NOFOLLOW | O_CREAT | O_TRUNC, 0640);
    *sessionfile = fdopen(fd, "w");
    if (fstat(fd, &filest) != 0) 
    {
	printf("\n");
	elog("Cannot stat session file :%s - probably a symlink attack!!!\n", fname);
	return hash_err;
    }
    if (filest.st_nlink > 1) 
    {
	printf("\n");
	elog("Hardlink attack attempted! File: %s Owner UID: %d! Please remove the hardlink manually!\n", fname, filest.st_uid);
	fclose(*sessionfile);
	return hash_err;
    }
    if (!(*sessionfile))
    {
	printf("\n");
	elog("Cannot open session file %s for writing (%s)!\n", fname, strerror(errno));
	return hash_err;
    }

    return hash_ok;
}


/* close session file */
void session_close_file(FILE *sessionfile)
{
    FILE *sf; // valgrind!
    uid_t uid;
    struct passwd *pwd;
    char fname[512];
    char fname2[512];
    int fd;
    struct stat filest;

    sf = sessionfile;
    fclose(sf);

    uid = getuid();
    pwd = getpwuid(uid);
    if (!pwd)
    {
	elog("Cannot get username for uid %d (%s)\n", (int)uid, strerror(errno));
	return;
    }
    snprintf(fname, 255, "/usr/share/hashkill/sessions/session.tmp", pwd->pw_name, getpid());
    snprintf(fname2, 255, "/usr/share/hashkill/sessions/%s-%d.session", pwd->pw_name, getpid());

    fd = open(fname2, O_WRONLY | O_NOFOLLOW | O_CREAT | O_TRUNC, 0640);
    sessionfile = fdopen(fd, "w");
    if (fstat(fd, &filest) != 0) 
    {
	printf("\n");
	elog("Cannot stat session file :%s - probably a symlink attack!!!\n", fname);
	return;
    }
    if (filest.st_nlink > 1) 
    {
	printf("\n");
	elog("Hardlink attack attempted! File: %s Owner UID: %d! Please remove the hardlink manually!\n", fname, filest.st_uid);
	fclose(sessionfile);
	return;
    }
    if (!(sessionfile))
    {
	printf("\n");
	elog("Cannot open session file %s for writing (%s)!\n", fname, strerror(errno));
	return;
    }
    endpwent();
    fclose(sessionfile);
    unlink(fname2);
    rename(fname, fname2);
    unlink(fname);
}



/* delete session file */
void session_unlink_file()
{
    struct passwd *pwd;
    char fname[255];
    int cnt = 0;
    
    pwd = getpwuid(getuid());
    if (!pwd)
    {
	elog("Cannot get username for uid %d (%s)\n", (int)getuid(), strerror(errno));
    }
    snprintf(fname, 255, "/usr/share/hashkill/sessions/%s-%d.session", pwd->pw_name, getpid());
    unlink(fname);
    while (session_argv[cnt])
    {
	free(session_argv[cnt]);
	cnt++;
    }
    endpwent();
}


/* Write hash list to session file */
hash_stat session_write_hashlist(FILE *sessionfile)
{
    struct hash_list_s *mylist;
    char *sectionheader = "<hash_list>";
    char *sectionfooter = "</hash_list>";
    char *newline = "\n";
    char *buff;
    
    if (!sessionfile)
    {
	return hash_err;
    }
    
    /* put section header */
    if (fputs(sectionheader, sessionfile)<0)
    {
	elog("Cannot write to session file : %s\n",strerror(errno));
	return hash_err;
    }
    fputs(newline, sessionfile);
    
    /* write out hash_list */
    pthread_mutex_lock(&listmutex);
    mylist = hash_list;
    while ((mylist)&&(mylist->username))
    {
	fputs(mylist->username, sessionfile);
	fputs(newline, sessionfile);
	buff = base64(mylist->hash, hash_ret_len);
	fputs(buff, sessionfile);
	free(buff);
	//fputs(mylist->hash, sessionfile);
	fputs(newline, sessionfile);
	fputs(mylist->salt, sessionfile);
	fputs(newline, sessionfile);
	buff = base64(mylist->salt2, hash_ret_len);
	fputs(buff, sessionfile);
	free(buff);

	//fputs(mylist->salt2, sessionfile);
	fputs(newline, sessionfile);
	mylist = mylist->next;
    }
    pthread_mutex_unlock(&listmutex);
    fputs(sectionfooter, sessionfile);
    fputs(newline, sessionfile);
    return hash_ok;
}



/* write cracked list to session file */
hash_stat session_write_crackedlist(FILE *sessionfile)
{
    struct hash_list_s *mylist;
    char *sectionheader = "<cracked_list>";
    char *sectionfooter = "</cracked_list>";
    char *newline = "\n";
    char *buff;
    
    if (!sessionfile)
    {
	return hash_err;
    }
    
    /* put section header */
    if (fputs(sectionheader, sessionfile)<0)
    {
	elog("Cannot write to session file : %s\n",strerror(errno));
	return hash_err;
    }
    fputs(newline, sessionfile);
    
    /* write out hash_list */
    pthread_mutex_lock(&crackedmutex);
    usleep(1000); // Stupid fix
    mylist = cracked_list;
    while (mylist)
    {
	fputs(mylist->username, sessionfile);
	fputs(newline, sessionfile);
	buff = base64(mylist->hash, hash_ret_len);
	fputs(buff, sessionfile);
	free(buff);
	fputs(newline, sessionfile);
	fputs(mylist->salt, sessionfile);
	fputs(newline, sessionfile);
	buff = base64(mylist->salt2, hash_ret_len);
	fputs(buff, sessionfile);
	free(buff);
	fputs(newline, sessionfile);
	mylist = mylist->next;
    }
    pthread_mutex_unlock(&crackedmutex);

    fputs(sectionfooter, sessionfile);
    fputs(newline, sessionfile);
    return hash_ok;
}



/* write dictionary attack parameters to session file */
hash_stat session_write_dictionary_parm(long filepos, char *dictfile, FILE *sessionfile)
{
    char *sectionheader = "<dictionary>";
    char *sectionfooter = "</dictionary>";
    char *newline = "\n";
    char intfpos[16];
    
    /* put section header */
    if (fputs(sectionheader, sessionfile)<0)
    {
	elog("Cannot write to session file : %s\n",strerror(errno));
	return hash_err;
    }
    fputs(newline, sessionfile);
    
    fputs(dictfile, sessionfile);
    fputs(newline, sessionfile);
    snprintf(intfpos, 16, "%lld", filepos);
    fputs(intfpos, sessionfile);
    fputs(newline, sessionfile);
    
    /* put section footer */
    if (fputs(sectionfooter, sessionfile)<0)
    {
	elog("Cannot write to session file : %s\n",strerror(errno));
	return hash_err;
    }
    fputs(newline, sessionfile);
    return hash_ok;
}



/* write bruteforce attack parameters to session file */
hash_stat session_write_bruteforce_parm(int start, int end, char *prefix, char *suffix, char *charset, int curlen, char *curstr, uint64_t progress, FILE *sessionfile)
{
    char *sectionheader = "<bruteforce>";
    char *sectionfooter = "</bruteforce>";
    char *newline = "\n";
    char intstrn[16];
    
    /* put section header */
    if (fputs(sectionheader, sessionfile)<0)
    {
	elog("Cannot write to session file : %s\n",strerror(errno));
	return hash_err;
    }
    fputs(newline, sessionfile);
    
    snprintf(intstrn, 8, "%d", start);
    fputs(intstrn, sessionfile);
    fputs(newline, sessionfile);
    snprintf(intstrn, 8, "%d", end);
    fputs(intstrn, sessionfile);
    fputs(newline, sessionfile);
    fputs(prefix, sessionfile);
    fputs(newline, sessionfile);
    fputs(suffix, sessionfile);
    fputs(newline, sessionfile);
    fputs(charset, sessionfile);
    fputs(newline, sessionfile);
    snprintf(intstrn, 8, "%d", curlen);
    fputs(intstrn, sessionfile);
    fputs(newline, sessionfile);
    fputs(curstr, sessionfile);
    fputs(newline, sessionfile);
    snprintf(intstrn, 16, "%lld", (long long)progress);
    fputs(intstrn, sessionfile);
    fputs(newline, sessionfile);

    /* put section footer */
    if (fputs(sectionfooter, sessionfile)<0)
    {
	elog("Cannot write to session file : %s\n",strerror(errno));
	return hash_err;
    }
    fputs(newline, sessionfile);
    return hash_ok;
}


/* write Markov attack parameters to session file */
hash_stat session_write_markov_parm(char *statfile, int threshold, int len, uint64_t count, uint64_t current_elem, char *current_str, FILE *sessionfile)
{
    char *sectionheader = "<markov>";
    char *sectionfooter = "</markov>";
    char *newline = "\n";
    char intstrn[16];
    
    /* put section header */
    if (fputs(sectionheader, sessionfile)<0)
    {
	elog("Cannot write to session file : %s\n",strerror(errno));
	return hash_err;
    }
    fputs(newline, sessionfile);
    
    fputs(statfile, sessionfile);
    fputs(newline, sessionfile);
    snprintf(intstrn, 8, "%d", threshold);
    fputs(intstrn, sessionfile);
    fputs(newline, sessionfile);
    snprintf(intstrn, 8, "%d", len);
    fputs(intstrn, sessionfile);
    fputs(newline, sessionfile);
    snprintf(intstrn, 8, "%lld", (long long)count);
    fputs(intstrn, sessionfile);
    fputs(newline, sessionfile);
    snprintf(intstrn, 8, "%lld", (long long)current_elem);
    fputs(intstrn, sessionfile);
    fputs(newline, sessionfile);
    fputs(current_str, sessionfile);
    fputs(newline, sessionfile);

    /* put section footer */
    if (fputs(sectionfooter, sessionfile)<0)
    {
	elog("Cannot write to session file : %s\n",strerror(errno));
	return hash_err;
    }
    fputs(newline, sessionfile);
    return hash_ok;
}




/* write hybrid attack parameters to session file */
hash_stat session_write_hybrid_parm(char *dictfile, int start, int end, char *charset, long current_pos, char *current_str, FILE *sessionfile)
{
    char *sectionheader = "<hybrid>";
    char *sectionfooter = "</hybrid>";
    char *newline = "\n";
    char intstrn[16];
    
    /* put section header */
    if (fputs(sectionheader, sessionfile)<0)
    {
	elog("Cannot write to session file : %s\n",strerror(errno));
	return hash_err;
    }
    fputs(newline, sessionfile);
    /* put dictionary file */
    fputs(dictfile, sessionfile);
    fputs(newline, sessionfile);
    /* put start/end */
    snprintf(intstrn, 8, "%d", start);
    fputs(intstrn, sessionfile);
    fputs(newline, sessionfile);
    snprintf(intstrn, 8, "%d", end);
    fputs(intstrn, sessionfile);
    fputs(newline, sessionfile);
    /* put charset */
    fputs(charset, sessionfile);
    fputs(newline, sessionfile);
    /* put current position */
    snprintf(intstrn, 16, "%lld", current_pos);
    fputs(intstrn, sessionfile);
    fputs(newline, sessionfile);
    fputs(current_str, sessionfile);
    fputs(newline, sessionfile);
    /* put section footer */
    if (fputs(sectionfooter, sessionfile)<0)
    {
	elog("Cannot write to session file : %s\n",strerror(errno));
	return hash_err;
    }
    fputs(newline, sessionfile);
    return hash_ok;
}


/* Put argv[] to session file */
void session_put_commandline(char *argv[])
{
    int cnt = 0;
    while ( (argv[cnt]) && (cnt <20))
    {
	session_argv[cnt] = malloc(strlen(argv[cnt])+10);
	strncpy(session_argv[cnt], argv[cnt], strlen(argv[cnt])); // thank you valgrind
	cnt++;
    }
    argv[cnt] = NULL;
}


/* Put general attack parameters to session file */
hash_stat session_write_parameters(char *plugin, attack_method_t attacktype, uint64_t progress, FILE *sessionfile)
{
    char *sectionheader = "<session>";
    char *sectionfooter = "</session>";
    char *newline = "\n";
    char *whitespace = " ";
    char intstrn[4];
    char buf[4096];

    struct tm *mytime;
    time_t myclock;
    int cnt=0;
    
    /* put section header */
    if (fputs(sectionheader, sessionfile)<0)
    {
	elog("Cannot write to session file : %s\n",strerror(errno));
	return hash_err;
    }
    fputs(newline, sessionfile);

    cnt = 0;
    /* put program parameters */
    while (session_argv[cnt] != NULL)
    {
	fputs(session_argv[cnt], sessionfile);
	fputs(whitespace, sessionfile);
	cnt++;
    }
    fputs(newline, sessionfile);
    
    /* put plugin name */
    fputs(plugin, sessionfile);
    fputs(newline, sessionfile);
    
    /* put attack type */
    snprintf(intstrn, 4, "%d", (int)attacktype);
    fputs(intstrn, sessionfile);
    fputs(newline, sessionfile);

    /* put progress(percent) */
    intstrn[0]=0; // VALGRIND!
    snprintf(intstrn, 4, "%d", (int)progress);
    fputs(intstrn, sessionfile);
    fputs(newline, sessionfile);

    /* put timestamp */
    (void)time(&myclock);
    mytime = localtime(&myclock);
    fputs(asctime(mytime), sessionfile);

    /* put cwd */
    getcwd((char *)&buf, 4096);
    fputs(buf, sessionfile);
    fputs(newline, sessionfile);

    /* put hash_len */
    snprintf(intstrn, 4, "%d", hash_ret_len);
    fputs(intstrn, sessionfile);
    fputs(newline, sessionfile);

    /* put hashlist file */
    fputs(hashlist_file, sessionfile);
    fputs(newline, sessionfile);


    
    /* put section footer */
    if (fputs(sectionfooter, sessionfile)<0)
    {
	elog("Cannot write to session file : %s\n",strerror(errno));
	return hash_err;
    }
    fputs(newline, sessionfile);
    return hash_ok;
}



/* Print to stdout sessions summary */
hash_stat print_sessions_summary(void)
{
    DIR *dir;
    FILE *sesfile;
    struct dirent *dentry;
    char sesname[1024];
    char readline[256];
    int attacktype;
    int progress;
    char plugin[256];
    char gmtime[256];
    char shortsesname[256];
    int cnt=0;
    
    dir=opendir("/usr/share/hashkill/sessions");
    if (!dir)
    {
        elog("Cannot open sessions dir: %s", "/usr/share/hashkill/sessions");
        return hash_err;
    }
    hlog("Sessions list: %s\n\n","");
    printf("Session name: \tSession ends at: \t\tSession type: \tProgress: \tPlugin: \n"
	    "--------------------------------------------------------------------------------------------\n");
    do
    {
        errno = 0;
        if ((dentry = readdir(dir)) != NULL)
        {
            if ((dentry->d_type == DT_REG) && (strstr(dentry->d_name, ".session")))
            {
                snprintf(sesname,1024,"/usr/share/hashkill/sessions/%s", dentry->d_name);
                /* Parse the <session>..</session> info */
                sesfile = fopen(sesname, "r");
                if (!sesfile)
                {
            	    goto next;
            	}
        	if (!fgets(readline, 256, sesfile))
        	{
        	    elog("Cannot read from session file : %s\n",sesname);
        	    goto next;
        	}
            	if (strcmp(readline,"<session>\n")!=0)
            	{
            	    elog("Malformed session file: %s\n",sesname);
            	    goto next;
            	}
            	/* skip command-line */
            	if (!fgets(readline, 256, sesfile))
        	{
        	    elog("Cannot read from session file : %s\n",sesname);
        	    goto next;
        	}
            	/* get plugin name */
            	if (!fgets(readline, 256, sesfile))
        	{
        	    elog("Cannot read from session file : %s\n",sesname);
        	    goto next;
        	}
        	snprintf(plugin,16,"%s",readline);
        	plugin[strlen(plugin)-1] = 0;

                /* get attack type */
                if (!fgets(readline, 256, sesfile))
        	{
        	    elog("Cannot read from session file : %s\n",sesname);
        	    goto next;
        	}
        	attacktype = atoi(readline);
                /* get attack progress */
                if (!fgets(readline, 256, sesfile))
        	{
        	    elog("Cannot read from session file : %s\n",sesname);
        	    goto next;
        	}
        	progress = atoi(readline);
                /* get timestamp */
                if (!fgets(readline, 256, sesfile))
        	{
        	    elog("Cannot read from session file : %s\n",sesname);
        	    goto next;
        	}
        	snprintf(gmtime,64,"%s",readline);
        	gmtime[strlen(gmtime)-1] = 0;
                /* get cwd */
                if (!fgets(readline, 256, sesfile))
        	{
        	    elog("Cannot read from session file : %s\n",sesname);
        	    goto next;
        	}
                /* skip */
                if (!fgets(readline, 256, sesfile))
        	{
        	    elog("Cannot read from session file : %s\n",sesname);
        	    goto next;
        	}
                /* get hashlist_file */
                if (!fgets(readline, 256, sesfile))
        	{
        	    elog("Cannot read from session file : %s\n",sesname);
        	    goto next;
        	}

        	/* get short session name */
        	cnt = 0;
        	while (dentry->d_name[cnt] != '.')
        	{
        	    shortsesname[cnt] = dentry->d_name[cnt];
        	    cnt++;
        	}
        	shortsesname[cnt] = 0;
        	
		/* Print out details */
		printf("\033[1;33m%s\033[1;0m \t",shortsesname);
		printf("%s \t",gmtime);
		switch (attacktype)
		{
		    case attack_method_dictionary: printf("Dictionary \t");break;
		    case attack_method_simple_bruteforce: printf("Bruteforce \t");break;
		    case attack_method_hybrid: printf("Hybrid \t\t");break;
		    case attack_method_markov: printf("Markov \t\t");break;
		    default: printf("Unknown \t");break;
                }
                if (progress > 100) printf("Unknown \t");
                else printf("%d%% \t\t",progress);
                printf("%s ",plugin);
                printf("\n");
                next:
    		if (sesfile) fclose(sesfile);
            }
        }
    } while (dentry != NULL);
    closedir(dir);
    printf("\n");
    return hash_ok;
}



/* Print to stdout detailed session summary */
hash_stat print_session_detailed(char *sessionname)
{
    FILE *sesfile;
    char sesname[1024];
    char readline[256];
    int attacktype;
    int progress;
    int cnt=0;
    char username[HASHFILE_MAX_LINE_LENGTH];
    char hash[HASHFILE_MAX_LINE_LENGTH];
    char salt[HASHFILE_MAX_LINE_LENGTH];
    int flag = 0;
    printf("\nDetailed information about session: %s\n"
	    "-----------------------------------------------\n",sessionname);
    snprintf(sesname,1024,"/usr/share/hashkill/sessions/%s.session", sessionname);
    /* Parse the <session>..</session> info */
    sesfile = fopen(sesname, "r");
    if (!sesfile)
    {
        elog("Cannot open session file : %s\n",sesname);
        return hash_err;
    }
    printf("Session file: \t%s\n",sesname);
    if (!fgets(readline, 256, sesfile))
    {
        elog("Cannot read from session file : %s\n",sesname);
        return hash_err;
    }
    /* get command line */
    if (!fgets(readline, 256, sesfile))
    {
        elog("Cannot read from session file : %s\n",sesname);
        return hash_err;
    }
    printf("Command line: \t%s",readline);
    /* get plugin */
    if (!fgets(readline, 256, sesfile))
    {
        elog("Cannot read from session file : %s\n",sesname);
        return hash_err;
    }
    printf("Plugin used: \t%s",readline);
    /* get attack type */
    if (!fgets(readline, 256, sesfile))
    {
        elog("Cannot read from session file : %s\n",sesname);
        return hash_err;
    }
    attacktype = atoi(readline);
    printf("Attack type: \t");
    switch (attacktype)
    {
	case attack_method_dictionary: printf("Dictionary\n");break;
	case attack_method_simple_bruteforce: printf("Bruteforce\n");break;
	case attack_method_hybrid: printf("Hybrid\n");break;
	default: printf("Unknown\n");
    }
    /* get attack progress */
    if (!fgets(readline, 256, sesfile))
    {
        elog("Cannot read from session file : %s\n",sesname);
        return hash_err;
    }
    progress = atoi(readline);
    if (progress>100) printf("Progress: \tUnknown\n");
    else printf("Progress: \t%d%%\n",progress);
    /* get timestamp */
    if (!fgets(readline, 256, sesfile))
    {
        elog("Cannot read from session file : %s\n",sesname);
        return hash_err;
    }
    printf("Session ends: \t%s",readline);
    /* get cwd */
    if (!fgets(readline, 256, sesfile))
    {
        elog("Cannot read from session file : %s\n",sesname);
        return hash_err;
    }
    /* skip */
    if (!fgets(readline, 256, sesfile))
    {
        elog("Cannot read from session file : %s\n",sesname);
        return hash_err;
    }

    /* skip */
    if (!fgets(readline, 256, sesfile))
    {
        elog("Cannot read from session file : %s\n",sesname);
        return hash_err;
    }

    /* skip section footer */
    if (!fgets(readline, 256, sesfile))
    {
        elog("Cannot read from session file : %s\n",sesname);
        return hash_err;
    }

    /* now parse bruteforce/dictionary parameters */
    if (!fgets(readline, 256, sesfile))
    {
        elog("Cannot read from session file : %s\n",sesname);
        return hash_err;
    }
    /* dictionary? */
    if (strcmp(readline,"<dictionary>\n")==0)
    {
	printf("\nDictionary attack parameters:\n\n");
	if (!fgets(readline, 256, sesfile))
	{
    	    elog("Cannot read from session file : %s\n",sesname);
    	    return hash_err;
	}
	printf("Dictionary file: %s",readline);
	/* skip position and section footer */
	if (!fgets(readline, 256, sesfile))
	{
    	    elog("Cannot read from session file : %s\n",sesname);
    	    return hash_err;
	}
	if (!fgets(readline, 256, sesfile))
	{
    	    elog("Cannot read from session file : %s\n",sesname);
    	    return hash_err;
	}
    }
    /* bruteforce : start/end/prefix/suffix/charset/<skip>/currentstr */
    else if (strcmp(readline,"<bruteforce>\n")==0)
    {
	printf("\nBruteforce attack parameters:\n"
		"------------------------------\n");
	if (!fgets(readline, 256, sesfile))
	{
    	    elog("Cannot read from session file : %s\n",sesname);
    	    return hash_err;
	}
	cnt = atoi(readline);
	printf("Bforce min: \t%d\n",cnt);
	if (!fgets(readline, 256, sesfile))
	{
    	    elog("Cannot read from session file : %s\n",sesname);
    	    return hash_err;
	}
	cnt = atoi(readline);
	printf("Bforce max: \t%d\n",cnt);
	if (!fgets(readline, 256, sesfile))
	{
    	    elog("Cannot read from session file : %s\n",sesname);
    	    return hash_err;
	}
	printf("Prefix: \t%s", readline);
	if (!fgets(readline, 256, sesfile))
	{
    	    elog("Cannot read from session file : %s\n",sesname);
    	    return hash_err;
	}
	printf("Suffix: \t%s", readline);
	if (!fgets(readline, 256, sesfile))
	{
    	    elog("Cannot read from session file : %s\n",sesname);
    	    return hash_err;
	}
	printf("Charset: \t%s", readline);
	/* skip curlen */
	if (!fgets(readline, 256, sesfile))
	{
    	    elog("Cannot read from session file : %s\n",sesname);
    	    return hash_err;
	}
	if (!fgets(readline, 256, sesfile))
	{
    	    elog("Cannot read from session file : %s\n",sesname);
    	    return hash_err;
	}
	printf("Current str: \t%s", readline);
	/* skip footer */
	if (!fgets(readline, 256, sesfile))
	{
    	    elog("Cannot read from session file : %s\n",sesname);
    	    return hash_err;
	}
	/* skip footer - once again*/
	fgets(readline, 256, sesfile);

    }
    /* hybrid : dictfile/start/end/charset/<skip>/currentstr */
    else if (strcmp(readline,"<hybrid>\n")==0)
    {
	printf("\nHybrid attack parameters:\n"
		"------------------------------\n");
	if (!fgets(readline, 256, sesfile))
	{
    	    elog("Cannot read from session file : %s\n",sesname);
    	    return hash_err;
	}
	printf("Dictionary: \t%s", readline);
	if (!fgets(readline, 256, sesfile))
	{
    	    elog("Cannot read from session file : %s\n",sesname);
    	    return hash_err;
	}
	cnt = atoi(readline);
	printf("Bforce min: \t%d\n",cnt);
	if (!fgets(readline, 256, sesfile))
	{
    	    elog("Cannot read from session file : %s\n",sesname);
    	    return hash_err;
	}
	cnt = atoi(readline);
	printf("Bforce max: \t%d\n",cnt);
	if (!fgets(readline, 256, sesfile))
	{
    	    elog("Cannot read from session file : %s\n",sesname);
    	    return hash_err;
	}
	printf("Charset: \t%s", readline);
	/* skip curlen */
	if (!fgets(readline, 256, sesfile))
	{
    	    elog("Cannot read from session file : %s\n",sesname);
    	    return hash_err;
	}
	/* get current str */
	if (!fgets(readline, 256, sesfile))
	{
    	    elog("Cannot read from session file : %s\n",sesname);
    	    return hash_err;
	}
	printf("Current str: \t%s", readline);
	/* skip footer */
	if (!fgets(readline, 256, sesfile))
	{
    	    elog("Cannot read from session file : %s\n",sesname);
    	    return hash_err;
	}
    }

    /* markov : set/threshold/len/count_max(skip)/count_current(skip)/currentstr */
    else if (strcmp(readline,"<markov>\n")==0)
    {
	printf("\nMarkov attack parameters:\n"
		"------------------------------\n");
	if (!fgets(readline, 256, sesfile))
	{
    	    elog("Cannot read from session file : %s\n",sesname);
    	    return hash_err;
	}
	printf("Statfile: \t%s", readline);
	if (!fgets(readline, 256, sesfile))
	{
    	    elog("Cannot read from session file : %s\n",sesname);
    	    return hash_err;
	}
	cnt = atoi(readline);
	printf("Threshold: \t%d\n",cnt);
	if (!fgets(readline, 256, sesfile))
	{
    	    elog("Cannot read from session file : %s\n",sesname);
    	    return hash_err;
	}
	cnt = atoi(readline);
	printf("Max. len: \t%d\n",cnt);
	/* skip 2 fields */
	if (!fgets(readline, 256, sesfile))
	{
    	    elog("Cannot read from session file : %s\n",sesname);
    	    return hash_err;
	}
	if (!fgets(readline, 256, sesfile))
	{
    	    elog("Cannot read from session file : %s\n",sesname);
    	    return hash_err;
	}
	/* skip footer */
	if (!fgets(readline, 256, sesfile))
	{
    	    elog("Cannot read from session file : %s\n",sesname);
    	    return hash_err;
	}
	if (!fgets(readline, 256, sesfile))
	{
    	    elog("Cannot read from session file : %s\n",sesname);
    	    return hash_err;
	}

    }


    /* unknown attack?!? */
    else
    {
	printf("Unknown attack type!\n");
	return hash_err;
    }

    /* print hashes list */
    printf("\nHashes list (username:hash:salt):\n---------------------------------\n");
    
    if (!fgets(readline, 256, sesfile))
    {
        elog("Cannot read from session file : %s\n",sesname);
        return hash_err;
    }
    if (strcmp(readline, "<hash_list>\n")!=0)
    {
        elog("Cannot read hash_list header from session file : %s (%s)\n",sesname, readline);
        return hash_err;
    }

    flag = 0;
    while (flag == 0)
    {
	if (!fgets(readline, 512, sesfile))
	{
    	    elog("Cannot read from session file : %s\n",sesname);
    	    return hash_err;
	}
	if (strcmp(readline, "</hash_list>\n") != 0)
	{
	    readline[strlen(readline)-1] = 0;
	    strcpy(username, readline);
	    if (!fgets(readline, 512, sesfile))
	    {
    		elog("Cannot read from session file : %s\n",sesname);
    		return hash_err;
	    }
	    readline[strlen(readline)-1] = 0;
	    strcpy(hash, readline);
	    if (!fgets(readline, 512, sesfile))
	    {
    		elog("Cannot read from session file : %s\n",sesname);
    		return hash_err;
	    }
	    readline[strlen(readline)-1] = 0;
	    strcpy(salt, readline);
	    if (!fgets(readline, 512, sesfile))
	    {
    		elog("Cannot read from session file : %s\n",sesname);
    		return hash_err;
	    }
	    printf("%s:%s:%s\n", username, hash, salt);
	}
	else flag = 1;
    }


    /* print cracked list */
    printf("\nCracked hashes list (username:hash:salt):\n---------------------------------\n");
    
    if (!fgets(readline, 256, sesfile))
    {
        elog("Cannot read from session file : %s\n",sesname);
        return hash_err;
    }
    if (strcmp(readline, "<cracked_list>\n")!=0)
    {
        elog("Cannot read hash_list header from session file : %s\n",sesname);
        return hash_err;
    }

    flag = 0;
    while (flag == 0)
    {
	if (!fgets(readline, 256, sesfile))
	{
    	    elog("Cannot read from session file : %s\n",sesname);
    	    return hash_err;
	}
	if (strcmp(readline, "</cracked_list>\n") != 0)
	{
	    readline[strlen(readline)-1] = 0;
	    strcpy(username, readline);
	    if (!fgets(readline, 512, sesfile))
	    {
    		elog("Cannot read from session file : %s\n",sesname);
    		return hash_err;
	    }
	    readline[strlen(readline)-1] = 0;
	    strcpy(hash, readline);
	    if (!fgets(readline, 512, sesfile))
	    {
    		elog("Cannot read from session file : %s\n",sesname);
    		return hash_err;
	    }
	    readline[strlen(readline)-1] = 0;
	    strcpy(salt, readline);
	    if (!fgets(readline, 512, sesfile))
	    {
    		elog("Cannot read from session file : %s\n",sesname);
    		return hash_err;
	    }
	    printf("%s:%s:%s\n", username, hash, salt);
	}
	else flag = 1;
    }
    printf("\n");
    fclose(sesfile);
    hlog("Session %s dumped successfully\n\n", sessionname);
}



/* 
    Restore a session
    Threads must be setup and spawned manually after that
*/
hash_stat session_restore(char *sessionname)
{
    FILE *sesfile;
    char sesname[1024];
    char readline[512];
    char username[HASHFILE_MAX_LINE_LENGTH];
    char hash[HASHFILE_MAX_LINE_LENGTH];
    char salt[HASHFILE_MAX_LINE_LENGTH];
    char salt2[HASHFILE_MAX_LINE_LENGTH];
    char *temp;
    int itemp;
    uid_t uid;
    struct passwd *pwd;
    char fname[255];
    int flag = 0;


    snprintf(sesname,1024,"/usr/share/hashkill/sessions/%s.session", sessionname);
    /* Parse the <session>..</session> info */
    sesfile = fopen(sesname, "r");
    if (!sesfile)
    {
        elog("Cannot open session file : %s\n",sesname);
        return hash_err;
    }
    hlog("Restoring session from: %s\n",sesname);
    if (!fgets(readline, 256, sesfile))
    {
        elog("Cannot read from session file : %s\n",sesname);
        return hash_err;
    }
    /* get command line */
    if (!fgets(readline, 256, sesfile))
    {
        elog("Cannot read from session file : %s\n",sesname);
        return hash_err;
    }
    /* get plugin */
    if (!fgets(readline, 256, sesfile))
    {
        elog("Cannot read from session file : %s\n",sesname);
        return hash_err;
    }
    readline[strlen(readline)-1] = 0;
    set_current_plugin(readline);
    if (load_plugin() == hash_err) exit(EXIT_FAILURE);

    /* get attack type */
    if (!fgets(readline, 256, sesfile))
    {
        elog("Cannot read from session file : %s\n",sesname);
        return hash_err;
    }
    attack_method  = atoi(readline);
    /* skip attack progress */
    if (!fgets(readline, 256, sesfile))
    {
        elog("Cannot read from session file : %s\n",sesname);
        return hash_err;
    }
    /* skip timestamp */
    if (!fgets(readline, 256, sesfile))
    {
        elog("Cannot read from session file : %s\n",sesname);
        return hash_err;
    }
    
    /* get cwd */
    if (!fgets(readline, 256, sesfile))
    {
        elog("Cannot read from session file : %s\n",sesname);
        return hash_err;
    }
    readline[strlen(readline)-1] = 0;
    if (chdir(readline) != 0)
    {
	elog("Cannot set working directory to %s\n",readline);
	return hash_err;
    }
    else
    {
	hlog("Change to working directory: %s\n",readline);
    }
    /* get hash_ret_len */
    if (!fgets(readline, 256, sesfile))
    {
        elog("Cannot read from session file : %s\n",sesname);
        return hash_err;
    }
    hash_ret_len  = atoi(readline);

    /* get hashlist_file */
    if (!fgets(readline, 256, sesfile))
    {
        elog("Cannot read from session file : %s\n",sesname);
        return hash_err;
    }
    readline[strlen(readline)-1] = 0;
    strcpy(hashlist_file, readline);
    if (hash_plugin_is_special()) load_hashes_file(hashlist_file);
    
    /* skip section footer */
    if (!fgets(readline, 256, sesfile))
    {
        elog("Cannot read from session file : %s\n",sesname);
        return hash_err;
    }

    /* now parse bruteforce/dictionary parameters */
    if (!fgets(readline, 256, sesfile))
    {
        elog("Cannot read from session file : %s\n",sesname);
        return hash_err;
    }
    /* dictionary? */
    if (strcmp(readline,"<dictionary>\n")==0)
    {
	if (!fgets(readline, 256, sesfile))
	{
    	    elog("Cannot read from session file : %s\n",sesname);
    	    return hash_err;
	}
	readline[strlen(readline)-1] = 0;
	strcpy(hash_dictionary_file, readline);
	/* skip position and section footer */
	if (!fgets(readline, 256, sesfile))
	{
    	    elog("Cannot read from session file : %s\n",sesname);
    	    return hash_err;
	}
	thread_set_dict_pos(atol(readline));
	
	if (!fgets(readline, 256, sesfile))
	{
    	    elog("Cannot read from session file : %s\n",sesname);
    	    return hash_err;
	}
    }
    /* bruteforce : start/end/prefix/suffix/charset/<skip>/currentstr */
    else if (strcmp(readline,"<bruteforce>\n")==0)
    {
	if (!fgets(readline, 256, sesfile))
	{
    	    elog("Cannot read from session file : %s\n",sesname);
    	    return hash_err;
	}
	bruteforce_start = atoi(readline);
	if (!fgets(readline, 256, sesfile))
	{
    	    elog("Cannot read from session file : %s\n",sesname);
    	    return hash_err;
	}
	bruteforce_end = atoi(readline);
	if (!fgets(readline, 256, sesfile))
	{
    	    elog("Cannot read from session file : %s\n",sesname);
    	    return hash_err;
	}
	readline[strlen(readline)-1] = 0;
	strcpy(bruteforce_prefix,readline);
	if (!fgets(readline, 256, sesfile))
	{
    	    elog("Cannot read from session file : %s\n",sesname);
    	    return hash_err;
	}
	readline[strlen(readline)-1] = 0;
	strcpy(bruteforce_suffix,readline);
	if (!fgets(readline, 256, sesfile))
	{
    	    elog("Cannot read from session file : %s\n",sesname);
    	    return hash_err;
	}
	readline[strlen(readline)-1] = 0;
	strcpy(bruteforce_charset,readline);
	/* skip curlen */
	if (!fgets(readline, 256, sesfile))
	{
    	    elog("Cannot read from session file : %s\n",sesname);
    	    return hash_err;
	}
	thread_set_bf_len(atoi(readline));
	if (!fgets(readline, 256, sesfile))
	{
    	    elog("Cannot read from session file : %s\n",sesname);
    	    return hash_err;
	}
	readline[strlen(readline)-1] = 0;
	thread_set_bf_str(readline);
	/* attack pos */
	if (!fgets(readline, 256, sesfile))
	{
    	    elog("Cannot read from session file : %s\n",sesname);
    	    return hash_err;
	}
	thread_set_bf_pos(atoll(readline));


	/* skip footer */
	if (!fgets(readline, 256, sesfile))
	{
    	    elog("Cannot read from session file : %s\n",sesname);
    	    return hash_err;
	}
    }

    /* hybrid : dictfile/start/end/charset/curpos/currentstr */
    else if (strcmp(readline,"<hybrid>\n")==0)
    {
	/* get dictfile */
	if (!fgets(readline, 256, sesfile))
	{
    	    elog("Cannot read from session file : %s\n",sesname);
    	    return hash_err;
	}
	readline[strlen(readline)-1] = 0;
	strcpy(hash_dictionary_file, readline);
	/* start/end */
	if (!fgets(readline, 256, sesfile))
	{
    	    elog("Cannot read from session file : %s\n",sesname);
    	    return hash_err;
	}
	bruteforce_start = atoi(readline);
	if (!fgets(readline, 256, sesfile))
	{
    	    elog("Cannot read from session file : %s\n",sesname);
    	    return hash_err;
	}
	bruteforce_end = atoi(readline);
	/* charset */
	if (!fgets(readline, 256, sesfile))
	{
    	    elog("Cannot read from session file : %s\n",sesname);
    	    return hash_err;
	}
	readline[strlen(readline)-1] = 0;
	strcpy(bruteforce_charset,readline);
	/* attack pos */
	if (!fgets(readline, 256, sesfile))
	{
    	    elog("Cannot read from session file : %s\n",sesname);
    	    return hash_err;
	}
	thread_set_dict_pos(atol(readline));
	/* skip curstr */
	if (!fgets(readline, 256, sesfile))
	{
    	    elog("Cannot read from session file : %s\n",sesname);
    	    return hash_err;
	}
	/* skip footer */
	if (!fgets(readline, 256, sesfile))
	{
    	    elog("Cannot read from session file : %s\n",sesname);
    	    return hash_err;
	}
    }

    /* markov : set/threshold/len/max/current_pos/currentstr */
    else if (strcmp(readline,"<markov>\n")==0)
    {
	if (!fgets(readline, 256, sesfile))
	{
    	    elog("Cannot read from session file : %s\n",sesname);
    	    return hash_err;
	}
	readline[strlen(readline)-1] = 0;
	strcpy(markov_statfile,readline);
	if (!fgets(readline, 256, sesfile))
	{
    	    elog("Cannot read from session file : %s\n",sesname);
    	    return hash_err;
	}
	markov_threshold = atoi(readline);
	if (!fgets(readline, 256, sesfile))
	{
    	    elog("Cannot read from session file : %s\n",sesname);
    	    return hash_err;
	}
	markov_max_len = atoi(readline);
	if (!fgets(readline, 256, sesfile))
	{
    	    elog("Cannot read from session file : %s\n",sesname);
    	    return hash_err;
	}
	uint64_t temp;
	int temp_i;
	temp = (uint64_t)atoll(readline);
	thread_set_markov_max_pos(temp);
	if (!fgets(readline, 256, sesfile))
	{
    	    elog("Cannot read from session file : %s\n",sesname);
    	    return hash_err;
	}
	temp = (uint64_t)atoll(readline);
	thread_set_markov_pos(temp);
	if (!fgets(readline, 256, sesfile))
	{
    	    elog("Cannot read from session file : %s\n",sesname);
    	    return hash_err;
	}
	readline[strlen(readline)-1] = 0;
	thread_set_markov_str(readline);
	thread_set_markov_len(strlen(readline));
	/* skip footer */
	if (!fgets(readline, 256, sesfile))
	{
    	    elog("Cannot read from session file : %s\n",sesname);
    	    return hash_err;
	}
    }



    /* unknown attack?!? */
    else
    {
	elog("Session load failed: Unknown attack type!\n%s","");
	return hash_err;
    }

    /* get hashes list */
    if (hash_plugin_is_special() == 0)
    {
	if (!fgets(readline, 256, sesfile))
	{
    	    elog("Cannot read from session file : %s\n",sesname);
    	    return hash_err;
	}
	if (strcmp(readline, "<hash_list>\n")!=0)
	{
    	    elog("Cannot read hash_list header from session file : %s\n",sesname);
    	    return hash_err;
	}

	flag = 0;
        while (flag == 0)
        {
    	    if (!fgets(readline, 256, sesfile))
	    {
    		elog("Cannot read from session file : %s\n",sesname);
    		return hash_err;
	    }
	    if (strcmp(readline, "</hash_list>\n") != 0)
	    {
		if (strlen(readline)>0) readline[strlen(readline)-1] = 0;
		strcpy(username, readline);
		if (!fgets(readline, 512, sesfile))
		{
    		    elog("Cannot read from session file : %s\n",sesname);
    		    return hash_err;
		}
		if (strlen(readline)>0) readline[strlen(readline)-1] = 0;
		itemp = strlen(readline);
		temp = unbase64(readline,itemp);
		memcpy(hash, temp , hash_ret_len);
		free(temp);
		if (!fgets(readline, 512, sesfile))
		{
    		    elog("Cannot read from session file : %s\n",sesname);
    		    return hash_err;
		}
		if (strlen(readline)>0) readline[strlen(readline)-1] = 0;
		strcpy(salt, readline);
		if (!fgets(readline, 512, sesfile))
		{
    		    elog("Cannot read from session file : %s\n",sesname);
    		    return hash_err;
		}
		if (add_hash_list(username, hash, salt, "") == hash_err) {printf("aa\n");exit(EXIT_FAILURE);}
	    }
	    else flag = 1;
	}

	/* get cracked list */
	if (!fgets(readline, 256, sesfile))
	{
    	    elog("Cannot read from session file : %s\n",sesname);
    	    return hash_err;
	}
	if (strcmp(readline, "<cracked_list>\n")!=0)
	{
    	    elog("Cannot read hash_list header from session file : %s\n",sesname);
    	    return hash_err;
	}
	flag = 0;
	while (flag == 0)
	{
	    if (!fgets(readline, 512, sesfile))
	    {
    		elog("Cannot read from session file : %s\n",sesname);
    		return hash_err;
	    }
	    if (strcmp(readline, "</cracked_list>\n") != 0)
	    {
		readline[strlen(readline)-1] = 0;
		strcpy(username, readline);
		if (!fgets(readline, 512, sesfile))
		{
    		    elog("Cannot read from session file : %s\n",sesname);
    		    return hash_err;
		}
		readline[strlen(readline)-1] = 0;
		itemp = strlen(readline);
		temp = unbase64(readline,itemp);
		memcpy(hash, temp ,hash_ret_len);
		free(temp);
		if (!fgets(readline, 512, sesfile))
		{
    		    elog("Cannot read from session file : %s\n",sesname);
    		    return hash_err;
		}
		readline[strlen(readline)-1] = 0;
		strcpy(salt, readline);
		if (!fgets(readline, 256, sesfile))
		{
    		    elog("Cannot read from session file : %s\n",sesname);
    		    return hash_err;
		}
		if (add_cracked_list(username, hash, salt, salt2) == hash_err) {printf("bb\n");exit(EXIT_FAILURE);}
	    }
	    else flag = 1;
	}

    }
    fclose(sesfile);
    /* rename session */
    uid = getuid();
    pwd = getpwuid(uid);
    if (!pwd)
    {
	elog("Cannot get username for uid %d (%s)\n", (int)uid, strerror(errno));
	return hash_err;
    }
    snprintf(fname, 255, "/usr/share/hashkill/sessions/%s.session", sessionname);
    unlink(fname);
    hlog("Session %s will be renamed to %s-%d\n", sessionname, pwd->pw_name, getpid());
    session_restore_flag = 1;
    endpwent();
    return hash_ok;
}

